package org.implab.gradle.containers.cli;

import java.io.File;
import java.io.IOException;
import java.nio.file.Files;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;
import org.gradle.api.logging.Logger;

public abstract class DockerTraits {

    public abstract Logger getLogger();

    public abstract Optional<File> getWorkingDir();

    public abstract String getCliCmd();

    boolean execute(ProcessBuilder builder, int code) throws IOException, InterruptedException {
        var proc = builder.start();
        return proc.waitFor() == code;
    }

    protected ProcessBuilder builder(String... args) {
        var command = new ArrayList<String>(args.length + 1);
        command.add(getCliCmd());
        Arrays.stream(args).forEach(command::add);

        var builder = new ProcessBuilder(command);

        getWorkingDir().ifPresent(builder::directory);
        return builder();
    }

    protected ProcessBuilder builder(List<String> command) {
        var builder = new ProcessBuilder(command);

        getWorkingDir().ifPresent(builder::directory);
        return builder();
    }

    public void build(List<String> args) {
        
    }

    public boolean imageExists(String imageId) throws InterruptedException, IOException {
        getLogger().info("Check image {} exists", imageId);

        var builder = builder("image", "inspect", "--format", "image-exists", imageId);

        return execute(builder, 0);
    }

    public boolean imageExists(File imageIdFile) {
        if (imageIdFile.exists()) {
            try {
                var imageId = Files.readString(imageIdFile.toPath());
                return imageExists(imageId);
            } catch (IOException | InterruptedException e) {
                getLogger().error("Failed to read imageId {}: {}", imageIdFile, e);
                return false;
            }
        }
        return false;
    }
}
